@@ -172,7 +172,7 @@
   </section>
 </chapter>
 
-   $Date: 2011-02-07 15:42:58 +0000 (Mon, 07 Feb 2011) $ $Revision: 32005 $
+   $Date: 2012-10-22 16:57:56 +0100 (Mon, 22 Oct 2012) $ $Revision: 35732 $
    */ 
 
 #ifndef	INCLUDED_SQLClient_H
@@ -181,11 +181,14 @@
 #import	<Foundation/NSArray.h>
 #import	<Foundation/NSObject.h>
 
-@class	GSCache;
+@class	NSCountedSet;
+@class	NSMapTable;
 @class	NSMutableDictionary;
 @class	NSMutableSet;
 @class	NSString;
 @class	NSThread;
+
+@class	GSCache;
 @class	SQLTransaction;
 
 /**
@@ -241,7 +244,7 @@ extern NSString * const SQLClientDidDisconnectNotification;
  * Returns the number of items in the record.<br />
  * Subclasses must implement this method.
  */
-- (unsigned) count;
+- (NSUInteger) count;
 
 /**
  * Return the record as a mutable dictionary with the keys as the
@@ -262,12 +265,12 @@ extern NSString * const SQLClientDidDisconnectNotification;
 /** <override-subclass />
  * Returns the key at the specified indes.<br />
  */
-- (NSString*) keyAtIndex: (unsigned)index;
+- (NSString*) keyAtIndex: (NSUInteger)index;
 
 /** <override-subclass />
  * Returns the object at the specified indes.<br />
  */
-- (id) objectAtIndex: (unsigned)index;
+- (id) objectAtIndex: (NSUInteger)index;
 
 /**
  * Returns the value of the named field.<br />
@@ -279,7 +282,7 @@ extern NSString * const SQLClientDidDisconnectNotification;
  * Replaces the value at the specified index.<br />
  * Subclasses must implement this method.
  */
-- (void) replaceObjectAtIndex: (unsigned)index withObject: (id)anObject;
+- (void) replaceObjectAtIndex: (NSUInteger)index withObject: (id)anObject;
 
 /**
  * Replaces the value of the named field.<br />
@@ -383,6 +386,8 @@ SQLCLIENT_PRIVATE
   GSCache		*_cache;	/** The cache for query results */
   NSThread		*_cacheThread;	/** Thread for cache queries */
   unsigned int		_connectFails;	/** The count of connection failures */
+  NSMapTable            *_observers;    /** Observations of async events */
+  NSCountedSet          *_names;        /** Track notification names */
   /** Allow for extensions by allocating memory and pointing to it from
    * the _extra ivar.  That way we can avoid binary incompatibility between
    * minor releases.
@@ -579,8 +584,11 @@ SQLCLIENT_PRIVATE
  *   [db execute: @"UPDATE ", table, @" SET Name = ",
  *     myName, " WHERE ID = ", myId, nil];
  * </example>
+ * Where the database backend support it, this method returns the count of
+ * the number of rows to which the operation applied.  Otherwise this
+ * returns -1.
  */
-- (void) execute: (NSString*)stmt,...;
+- (NSInteger) execute: (NSString*)stmt,...;
 
 /**
  * Takes the statement and substitutes in values from
@@ -594,9 +602,12 @@ SQLCLIENT_PRIVATE
  * the results of the -quote: method.<br />
  * The markup format may also be {key?default} where <em>default</em>
  * is a string to be used if there is no value for the <em>key</em>
- * in the dictionary.
+ * in the dictionary.<br />
+ * Where the database backend support it, this method returns the count of
+ * the number of rows to which the operation applied.  Otherwise this
+ * returns -1.
  */
-- (void) execute: (NSString*)stmt with: (NSDictionary*)values;
+- (NSInteger) execute: (NSString*)stmt with: (NSDictionary*)values;
 
 /**
  * Calls -initWithConfiguration:name: passing a nil reference name.
@@ -635,6 +646,14 @@ SQLCLIENT_PRIVATE
 - (id) initWithConfiguration: (NSDictionary*)config
 			name: (NSString*)reference;
 
+/** Two clients are considered equal if they refer to the same database
+ * and are logged in as the same database user using the same protocol.
+ * These are the general criteria for transactions to be compatoible so
+ * that an SQLTransaction object generated by one client can be used by
+ * the other.
+ */
+- (BOOL) isEqual: (id)other;
+
 /**
  * Return the state of the flag indicating whether the library thinks
  * a transaction is in progress.  This flag is normally maintained by
@@ -811,9 +830,10 @@ SQLCLIENT_PRIVATE
 /**
  * Calls -backendExecute: in a safe manner.<br />
  * Handles locking.<br />
- * Maintains -lastOperation date.
+ * Maintains -lastOperation date.<br />
+ * Returns the result of -backendExecute:
  */
-- (void) simpleExecute: (NSArray*)info;
+- (NSInteger) simpleExecute: (NSArray*)info;
 
 /**
  * Calls -simpleQuery:recordType:listType: with the default record class
@@ -925,8 +945,12 @@ SQLCLIENT_PRIVATE
  * <p>Application code must <em>not</em> call this method directly, it is
  * for internal use only.
  * </p>
+ * <p>Where the database backend support it, this method returns the count of
+ * the number of rows to which the operation applied.  Otherwise this
+ * returns -1.
+ * </p>
  */
-- (void) backendExecute: (NSArray*)info;
+- (NSInteger) backendExecute: (NSArray*)info;
 
 /** <override-subclass />
  * <p>Perform arbitrary query <em>which returns values.</em>
@@ -981,6 +1005,47 @@ SQLCLIENT_PRIVATE
  */
 - (NSMutableArray*) backendQuery: (NSString*)stmt;
 
+/** <override-subclass />
+ * Called to enable asynchronous notification of database events using the
+ * specified name (which must be a valid identifier consisting of ascii
+ * letters, digits, and underscore characters, starting with a letter).
+ * Names are not case sensitive (so AAA is the same as aaa).<br />
+ * Repeated calls to list on the same name should be treated as a single
+ * call.<br />
+ * The backend is responsible for implicitly unlistening when a connection
+ * is closed.<br />
+ * There is a default implementation which does nothing ... for backends
+ * which don't support asynchronous notifications.<br />
+ * If a backend <em>does</em> support asynchronous notifications,
+ * it should do so by posting NSNotification instances to
+ * [NSNotificationCenter defaultCenter] using the SQLClient instance as
+ * the notification object and supplying any payload as a string using
+ * the 'Payload' key in the NSNotification userInfo dictionary.
+ * The userInfo dictionary should also contain a boolean (NSNumber) value,
+ * using the 'Local' key, to indicate whether the notification was sent by
+ * the current SQLClient instance or by some other client/
+ */
+- (void) backendListen: (NSString*)name;
+
+/** <override-subclass />
+ * The backend should implement this to send asynchronous notifications
+ * to anything listening for them. The name of the notification is an
+ * SQL identifier used for listening for the asynchronous data.<br />
+ * The payload string may be nil if no additional information is
+ * needed in the notification.
+ */
+- (void) backendNotify: (NSString*)name payload: (NSString*)more;
+
+/** <override-subclass />
+ * Called to disable asynchronous notification of database events using the
+ * specified name.  This has no effect if the name has not been used in an
+ * earlier call to -backendListen:, or if the name has already been
+ * unlistened since the last call to listen. on it.<br />
+ * There is a default implementation which does nothing ... for backends
+ * which don't support asynchronous notifications.
+ */
+- (void) backendUnlisten: (NSString*)name;
+
 /** <override-subclass />
  * This method is <em>only</em> for the use of the
  * -insertBLOBs:intoStatement:length:withMarker:length:giving:
@@ -1030,6 +1095,41 @@ SQLCLIENT_PRIVATE
 
 @end
 
+/**
+ * This category contains methods for asynchronous notification of 
+ * events via the database (for those database backends which support
+ * it: currently only PostgreSQL).
+ */
+@interface	SQLClient (Notifications)
+/** Adds anObserver to receive notifications when the backend database
+ * server sends an asynchronous event identified by the specified name
+ * (which must be a valid database identifier).<br />
+ * When a notification (NSNotification instance) is received by the method
+ * specified by aSelector, its <em>object</em> will be the SQLClient
+ * instance to which anObserver was added and its userInfo dictionary
+ * will contain the key 'Local' and possibly the key 'Payload'.<br />
+ * If the 'Local' value is the boolean YES, the notification originated
+ * as an action by this SQLClient instance.<br />
+ * If the 'Payload' value is not nil, then it is a string providing extra
+ * information about the notification.
+ */
+- (void) addObserver: (id)anObserver
+            selector: (SEL)aSelector
+                name: (NSString*)name;
+
+/** Posts a notification via the dastabase.  The name is an SQL identifier
+ * (for which observers may have registered) and the extra payload
+ * information may be nil if not required.
+ */
+- (void) postNotificationName: (NSString*)name payload: (NSString*)more;
+
+/** Removes anObserver as an observer for asynchronous notifications from
+ * the database server.  If name is omitted, the observer will be removed
+ * for all names.
+ */
+- (void) removeObserver: (id)anObserver name: (NSString*)name;
+@end
+
 /**
  * This category contains convenience methods including those for
  * frequently performed database operations ... message logging etc.
@@ -1306,7 +1406,7 @@ SQLCLIENT_PRIVATE
  * which have been added to the receiver.  For a count of the total number
  * of statements, use the -totalCount method.
  */
-- (unsigned) count;
+- (NSUInteger) count;
 
 /**
  * Returns the database client with which this instance operates.<br />
